// png2bmp.cpp : main project file.

#include "stdafx.h"

using namespace System;
using namespace System::IO;
using namespace System::Drawing;

// Writes any number of bytes to the stream
void WriteBytes(FileStream ^f, unsigned char *x, unsigned int len) {
	for (unsigned int i = 0; i < len; i++) {
		f->WriteByte(x[i]);
	}
}

// Writes a short to the stream
void WriteShort(FileStream ^f, short x) {
	WriteBytes(f, (unsigned char *)&x, 2);
}

// Writes a long to the stream
void WriteLong(FileStream ^f, long x) {
	WriteBytes(f, (unsigned char *)&x, 4);
}

int main(array<System::String ^> ^args) {
	if (args->Length < 1) {
		Console::WriteLine(L"You need to include at least one GIF, JPEG, PNG, TIFF, WMF, EMF, BMP, or ICO file on the command line");
	}
	for (int i = 0; i < args->Length; i++) {
		try {
			// Open the source image
			Bitmap ^bmp = gcnew Bitmap(args[i]);

			// Get the name of the new file (replace extension with .bmp)
			String ^name = args[i];
			int dot = name->LastIndexOf(L".");
			if (dot > 0)
				name = name->Remove(dot);
			name += L".bmp";

			// Calculate the size of the bitmap data in the image
			unsigned long bmpSize = bmp->Width*bmp->Height*4;
			
			// Open the BMP file
			FileStream ^out = gcnew FileStream(name, FileMode::Create, FileAccess::Write);

			// Write BMP Magic Number
			out->WriteByte('B');
			out->WriteByte('M');

			// Write BMP File Header
			WriteLong(out, 54+bmpSize);	// Size of the entire file
			WriteLong(out, 0);			// Reserved WORDs 1 and 2
			WriteLong(out, 54);			// Bitmap data start (always 54)

			// Write Bitmap data header
			WriteLong(out, 40);			// Header size (40 for Windows V3 / BITMAPINFOHEADER)
			WriteLong(out, bmp->Width);	// Image width
			WriteLong(out, bmp->Height);// Image height
			WriteShort(out, 1);			// Number of color planes (must be 1)
			WriteShort(out, 32);		// Bits per pixel (32 for full color and alpha images)
			WriteLong(out, 0);			// Compression type (0 for no compression)
			WriteLong(out, bmpSize);	// Size of the bitmap data
			WriteLong(out, 2835);		// Horizontal resolution of image (2835 px/m is typical)
			WriteLong(out, 2835);		// Vertical resolution of image (2835 px/m is typical)
			WriteLong(out, 0);			// Number of colors in palette (0 for no palette)
			WriteLong(out, 0);			// Number of important colors in palette (0 for all colors are important)

			// Write bitmap data
			out->Seek(54, SeekOrigin::Begin);
			for (int y = bmp->Height-1; y >= 0; y--) {
				for (int x = 0; x < bmp->Width; x++) {
					Color c = bmp->GetPixel(x, y);
					unsigned char px[4] = { c.B, c.G, c.R, c.A }; // BMP file is backwards (BGRA instead of ARGB)
					px[0] = px[0] * px[3] / 255; // We must pre-multiply the alpha values
					px[1] = px[1] * px[3] / 255;
					px[2] = px[2] * px[3] / 255;
					WriteBytes(out, px, 4); // Write the pixel
				}
			}

			out->Close();
		} catch (Exception ^ex) {
			Console::Error->WriteLine(L"There was a problem converting "+args[i]+": "+ex->Message);
		}
	}
	
    return 0;
}
